      program getsym

c Program to extract spacegroup related data.
c The subroutines; gt_real, input, is_char, is_int, mult
c push and upper can be found in the source code of SFTOOLS
c
c written by Bart Hazes, version January 11, 1995

      implicit none

      integer maxop, maxgrp
      parameter (maxop=45, maxgrp=70)

c space group related variables
      integer spnum,klass,nsym,nprim
      real symops(4,4,96), cell(6)
      character spgnam*10,centr*1,cr_syst(16)*12

c general variables
      character spgr_id*12, key*10, card*200, option*80, frmat*80
      character format*10
      integer l, m, n, nop, newsym, nspgrp, ops(7)
      real symmat(4,4), orimat(4,4), gt_real, cosa, cosb, cosg, celvol
      real deg2rad
      logical input, present

c data statement variables
      real g(12,maxop), celres(6,16)
      character*36 syminf(maxgrp), pgnams(16)*5, upp*25

c list of symmetry generators for each spacegroup (no centering)
c content = spacegroup number, spacegroup name, 5 spacegroup generators
c negative spacegroup numbers represent alternative settings
      data syminf(  1) /'    1 P1           0   0   0   0   0'/
      data syminf(  2) /'    3 P2           1   0   0   0   0'/
      data syminf(  3) /'   -3 P112         2   0   0   0   0'/
      data syminf(  4) /'    4 P21          3   0   0   0   0'/
      data syminf(  5) /'   -4 P1121        4   0   0   0   0'/
      data syminf(  6) /'    5 C2           1   0   0   0   0'/
      data syminf(  7) /'   -5 A112         2   0   0   0   0'/
      data syminf(  8) /'   16 P222         2   1   0   0   0'/
      data syminf(  9) /'   17 P2221        4   5   0   0   0'/
      data syminf( 10) /'   18 P21212       2   6   0   0   0'/
      data syminf( 11) /'   19 P212121      7   8   0   0   0'/
      data syminf( 12) /'   20 C2221        4   5   0   0   0'/
      data syminf( 13) /'   21 C222         2   1   0   0   0'/
      data syminf( 14) /'   22 F222         2   1   0   0   0'/
      data syminf( 15) /'   23 I222         2   1   0   0   0'/
      data syminf( 16) /'   24 I212121      7   8   0   0   0'/
      data syminf( 17) /'   75 P4           9   0   0   0   0'/
      data syminf( 18) /'   76 P41         10   0   0   0   0'/
      data syminf( 19) /'   77 P42         11   0   0   0   0'/
      data syminf( 20) /'   78 P43         12   0   0   0   0'/
      data syminf( 21) /'   79 I4           9   0   0   0   0'/
      data syminf( 22) /'   80 I41         13   0   0   0   0'/
      data syminf( 23) /'   89 P422         9   1   0   0   0'/
      data syminf( 24) /'   90 P4212       14   6   0   0   0'/
      data syminf( 25) /'   91 P4122       10   1   0   0   0'/
      data syminf( 26) /'   92 P41212      15  16   0   0   0'/
      data syminf( 27) /'   93 P4222       11   1   0   0   0'/
      data syminf( 28) /'   94 P42212      17  18   0   0   0'/
      data syminf( 29) /'   95 P4322       12   1   0   0   0'/
      data syminf( 30) /'   96 P43212      19  20   0   0   0'/
      data syminf( 31) /'   97 I422         9   1   0   0   0'/
      data syminf( 32) /'   98 I4122       13  21   0   0   0'/
      data syminf( 33) /'  143 P3          22   0   0   0   0'/
      data syminf( 34) /'  144 P31         23   0   0   0   0'/
      data syminf( 35) /'  145 P32         24   0   0   0   0'/
      data syminf( 36) /'  146 R3          22   0   0   0   0'/
      data syminf( 37) /' -146 R3R         25   0   0   0   0'/
      data syminf( 38) /'  149 P312        22  26   0   0   0'/
      data syminf( 39) /'  150 P321        22  27   0   0   0'/
      data syminf( 40) /'  151 P3112       23  28   0   0   0'/
      data syminf( 41) /'  152 P3121       23  27   0   0   0'/
      data syminf( 42) /'  153 P3212       24  29   0   0   0'/
      data syminf( 43) /'  154 P3221       24  27   0   0   0'/
      data syminf( 44) /'  155 R32         22  27   0   0   0'/
      data syminf( 45) /' -155 R32R        25  26   0   0   0'/
      data syminf( 46) /'  168 P6          30   0   0   0   0'/
      data syminf( 47) /'  169 P61         31   0   0   0   0'/
      data syminf( 48) /'  170 P65         32   0   0   0   0'/
      data syminf( 49) /'  171 P62         33   0   0   0   0'/
      data syminf( 50) /'  172 P64         34   0   0   0   0'/
      data syminf( 51) /'  173 P63         35   0   0   0   0'/
      data syminf( 52) /'  177 P622        30  27   0   0   0'/
      data syminf( 53) /'  178 P6122       31  36   0   0   0'/
      data syminf( 54) /'  179 P6522       32  37   0   0   0'/
      data syminf( 55) /'  180 P6222       33  37   0   0   0'/
      data syminf( 56) /'  181 P6422       34  36   0   0   0'/
      data syminf( 57) /'  182 P6322       35  27   0   0   0'/
      data syminf( 58) /'  195 P23          2   1  25   0   0'/
      data syminf( 59) /'  196 F23          2   1  25   0   0'/
      data syminf( 60) /'  197 I23          2   1  25   0   0'/
      data syminf( 61) /'  198 P213         7   8  25   0   0'/
      data syminf( 62) /'  199 I213         7   8  25   0   0'/
      data syminf( 63) /'  207 P432         9   1  25   0   0'/
      data syminf( 64) /'  208 P4232       17   1  25   0   0'/
      data syminf( 65) /'  209 F432         9   1  25   0   0'/
      data syminf( 66) /'  210 F4132       38   6  25   0   0'/
      data syminf( 67) /'  211 I432         9   1  25   0   0'/
      data syminf( 68) /'  212 P4332       39   8  25   0   0'/
      data syminf( 69) /'  213 P4132       40   8  25   0   0'/
      data syminf( 70) /'  214 I4132       40   8  25   0   0'/

c the matrices for each generator
      data (g(k,  1),k=1,12) /-1, 0, 0, 0, 0, 1, 0, 0, 0, 0,-1, 0/
      data (g(k,  2),k=1,12) /-1, 0, 0, 0, 0,-1, 0, 0, 0, 0, 1, 0/
      data (g(k,  3),k=1,12) /-1, 0, 0, 0, 0, 1, 0,12, 0, 0,-1, 0/
      data (g(k,  4),k=1,12) /-1, 0, 0, 0, 0,-1, 0, 0, 0, 0, 1,12/
      data (g(k,  5),k=1,12) /-1, 0, 0, 0, 0, 1, 0, 0, 0, 0,-1,12/
      data (g(k,  6),k=1,12) /-1, 0, 0,12, 0, 1, 0,12, 0, 0,-1, 0/
      data (g(k,  7),k=1,12) /-1, 0, 0,12, 0,-1, 0, 0, 0, 0, 1,12/
      data (g(k,  8),k=1,12) /-1, 0, 0, 0, 0, 1, 0,12, 0, 0,-1,12/
      data (g(k,  9),k=1,12) / 0,-1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0/
      data (g(k, 10),k=1,12) / 0,-1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 6/
      data (g(k, 11),k=1,12) / 0,-1, 0, 0, 1, 0, 0, 0, 0, 0, 1,12/
      data (g(k, 12),k=1,12) / 0,-1, 0, 0, 1, 0, 0, 0, 0, 0, 1,18/
      data (g(k, 13),k=1,12) / 0,-1, 0, 0, 1, 0, 0,12, 0, 0, 1, 6/
      data (g(k, 14),k=1,12) / 0,-1, 0,12, 1, 0, 0,12, 0, 0, 1, 0/
      data (g(k, 15),k=1,12) / 0,-1, 0,12, 1, 0, 0,12, 0, 0, 1, 6/
      data (g(k, 16),k=1,12) /-1, 0, 0,12, 0, 1, 0,12, 0, 0,-1, 6/
      data (g(k, 17),k=1,12) / 0,-1, 0,12, 1, 0, 0,12, 0, 0, 1,12/
      data (g(k, 18),k=1,12) /-1, 0, 0,12, 0, 1, 0,12, 0, 0,-1,12/
      data (g(k, 19),k=1,12) / 0,-1, 0,12, 1, 0, 0,12, 0, 0, 1,18/
      data (g(k, 20),k=1,12) /-1, 0, 0,12, 0, 1, 0,12, 0, 0,-1,18/
      data (g(k, 21),k=1,12) /-1, 0, 0,12, 0, 1, 0, 0, 0, 0,-1,18/
      data (g(k, 22),k=1,12) / 0,-1, 0, 0, 1,-1, 0, 0, 0, 0, 1, 0/
      data (g(k, 23),k=1,12) / 0,-1, 0, 0, 1,-1, 0, 0, 0, 0, 1, 8/
      data (g(k, 24),k=1,12) / 0,-1, 0, 0, 1,-1, 0, 0, 0, 0, 1,16/
      data (g(k, 25),k=1,12) / 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 0/
      data (g(k, 26),k=1,12) / 0,-1, 0, 0,-1, 0, 0, 0, 0, 0,-1, 0/
      data (g(k, 27),k=1,12) / 0, 1, 0, 0, 1, 0, 0, 0, 0, 0,-1, 0/
      data (g(k, 28),k=1,12) / 0,-1, 0, 0,-1, 0, 0, 0, 0, 0,-1,16/
      data (g(k, 29),k=1,12) / 0,-1, 0, 0,-1, 0, 0, 0, 0, 0,-1, 8/
      data (g(k, 30),k=1,12) / 0, 1, 0, 0,-1, 1, 0, 0, 0, 0, 1, 0/
      data (g(k, 31),k=1,12) / 0, 1, 0, 0,-1, 1, 0, 0, 0, 0, 1,20/
      data (g(k, 32),k=1,12) / 0, 1, 0, 0,-1, 1, 0, 0, 0, 0, 1, 4/
      data (g(k, 33),k=1,12) / 0, 1, 0, 0,-1, 1, 0, 0, 0, 0, 1,16/
      data (g(k, 34),k=1,12) / 0, 1, 0, 0,-1, 1, 0, 0, 0, 0, 1, 8/
      data (g(k, 35),k=1,12) / 0, 1, 0, 0,-1, 1, 0, 0, 0, 0, 1,12/
      data (g(k, 36),k=1,12) / 0, 1, 0, 0, 1, 0, 0, 0, 0, 0,-1, 8/
      data (g(k, 37),k=1,12) / 0, 1, 0, 0, 1, 0, 0, 0, 0, 0,-1,16/
      data (g(k, 38),k=1,12) / 0,-1, 0,18, 1, 0, 0,18, 0, 0, 1, 6/
      data (g(k, 39),k=1,12) / 0,-1, 0,18, 1, 0, 0, 6, 0, 0, 1,18/
      data (g(k, 40),k=1,12) / 0,-1, 0, 6, 1, 0, 0,18, 0, 0, 1, 6/
c centering operators
      data (g(k, 41),k=1,12) / 1, 0, 0, 0, 0, 1, 0,12, 0, 0, 1,12/
      data (g(k, 42),k=1,12) / 1, 0, 0,12, 0, 1, 0, 0, 0, 0, 1,12/
      data (g(k, 43),k=1,12) / 1, 0, 0,12, 0, 1, 0,12, 0, 0, 1, 0/
      data (g(k, 44),k=1,12) / 1, 0, 0,12, 0, 1, 0,12, 0, 0, 1,12/
      data (g(k, 45),k=1,12) / 1, 0, 0,16, 0, 1, 0, 8, 0, 0, 1, 8/

c point group names according to CCP4
      data pgnams / 'PG1'  ,'PG2'  ,'PG222', 'PG4'  ,'PG422', 'PG3',
     $              'PG3'  ,'PG312','PG321', 'PG321','PG6'  ,
     $              'PG622','PG23' ,'PG432', 'PG2'  ,'PG2'/

c crystal systems for each crystal klass
      data cr_syst /'triclinic', 'monoclinic', 'orthorhombic',
     $              'tetragonal', 'tetragonal', 'trigonal',
     $              'trigonal', 'trigonal', 'trigonal',
     $              'trigonal', 'hexagonal', 'hexagonal',
     $              'cubic', 'cubic', 'monoclinic', 'monoclinic'/

c cell parameter restrictions for the 16 crystal classes (a la BIOMOL)
      data celres/  -1.,  -2.,  -3.,  -4.,  -5.,  -6.,
     $              -1.,  -2.,  -3.,  90.,  -5.,  90.,
     $              -1.,  -2.,  -3.,  90.,  90.,  90.,
     $              -1.,  -1.,  -3.,  90.,  90.,  90.,
     $              -1.,  -1.,  -3.,  90.,  90.,  90.,
     $              -1.,  -1.,  -3.,  90.,  90., 120.,
     $              -1.,  -1.,  -1.,  -4.,  -4.,  -4.,
     $              -1.,  -1.,  -3.,  90.,  90., 120.,
     $              -1.,  -1.,  -3.,  90.,  90., 120.,
     $              -1.,  -1.,  -1.,  -4.,  -4.,  -4.,
     $              -1.,  -1.,  -3.,  90.,  90., 120.,
     $              -1.,  -1.,  -3.,  90.,  90., 120.,
     $              -1.,  -1.,  -1.,  90.,  90.,  90.,
     $              -1.,  -1.,  -1.,  90.,  90.,  90.,
     $              -1.,  -2.,  -3.,  -4.,  90.,  90.,
     $              -1.,  -2.,  -3.,  90.,  90.,  -6. /

      data upp / 'ABCFIPRABCFIPR0123456789-' /

      present = .true.
      deg2rad = asin(1.0)/90.0

      write(*,*)'Content-type: text/plain'
      write(*,*)

      read(*,'(A200)')card
      do 50, l=1,200
        if(card(l:l) .eq. '=' .or.
     $     card(l:l) .eq. '&' .or.
     $     card(l:l) .eq. '+' .or.
     $     ichar(card(l:l)) .eq. 13) card(l:l) = ' '
   50 continue

      format='GENERIC'
  100 if(input(card,option,frmat).eq.present)then
        key=option(1:10)
        if(input(card,option,frmat).ne.present) stop
        if(key.eq.'SPACEGROUP')then
          spgnam=option(1:10)
        else if(key.eq.'CEL1')then
          cell(1) = gt_real(option)
        else if(key.eq.'CEL2')then
          cell(2) = gt_real(option)
        else if(key.eq.'CEL3')then
          cell(3) = gt_real(option)
        else if(key.eq.'CEL4')then
          cell(4) = gt_real(option)
        else if(key.eq.'CEL5')then
          cell(5) = gt_real(option)
        else if(key.eq.'CEL6')then
          cell(6) = gt_real(option)
        else if(key.eq.'FORMAT')then
          format=option(1:10)
        endif
        goto 100
      endif

c copy spacegroup identifier such that there is a leading and trailing
c space and all characters are upper case. All space parenthesis etc are
c stripped of
      spgr_id=' '
      n = 1
      do 150,l=1,10
        m=index('ABCFIPRabcfipr0123456789-',spgnam(l:l))
        if(m.ne.0)then
          n=n+1
          spgr_id(n:n)=upp(m:m)
        endif
  150 continue
      n=n+1
      spgr_id(n:n)=' '

c locate the specified space group identifier in the syminf array
      do 200, l=1,maxgrp
        if(index(syminf(l)(1:15),spgr_id(1:n)).ne.0)then
          nspgrp=l
          goto 300
        endif
  200 continue
c Oops typo ?
      if(spgr_id(1:3).ne.' P ')then
        write(*,*)' WARNING: illegal spacegroup identifier'
      endif
      write(*,*)
      write(*,*)' Use one of the following names or numbers'
      write(*,*)
      write(*,'(4(a16,3x))')(syminf(l)(1:16),l=1,maxgrp)
      write(*,*)
      write(*,*)' Click on <BACK> to return to the input form'
      stop

c OK we found the space group, now read the symmetry generators (ops)
  300 read(syminf(nspgrp),'(i5,1x,a10,5i4)')spnum,spgnam,
     $  (ops(l),l=1,5)
      ops(6)=0
      ops(7)=0

c determine the crystal class based on the space group number and for
c monoclinic and rhombohedral use the cell parameters to distinguish settings
      if(spnum.eq.1)then
        klass = 1
      else if(spnum.ge.3 .and. spnum.le.5)then
        if(cell(4).ne.90.0 .and. cell(4).ne.0.0)then
          write(*,*)' !!! Error, monoclinic A unique, not supported !!!'
          stop
        else if(cell(6).ne.90.0 .and. cell(6).ne.0.0 .and.
     $         (cell(5).eq.90.0 .or.  cell(5).eq.0.0))then
          write(*,*)' !!! Warning, monoclinic C unique selected !!!'
          nspgrp=nspgrp+1
          goto 300
        endif
        if(cell(5).eq.90.0 .or. cell(5).eq.0.0)then
          write(*,*)' !!! WARNING, space group is ambiguous !!!'
          write(*,*)
          write(*,*)' monoclinic with B-unique has been selected'
          write(*,*)' use the negative space group number if you wish'
          write(*,*)' to work with C-unique (e.g. -3, -4 or -5)'
        endif
        klass = 2
      else if(16.le.spnum .and. spnum.le.24)then
        klass=3
      else if(75.le.spnum .and. spnum.le.80)then
        klass=4
      else if(89.le.spnum .and. spnum.le.98)then
        klass=5
      else if(143.le.spnum .and. spnum.le.145)then
        klass=6
      else if(146.eq.spnum)then
        if(cell(5) .eq. cell(6) .and. cell(5).ne.0.0)then
          write(*,*)' !!! Warning, Rhombohedral setting selected !!!'
          nspgrp=nspgrp+1
          goto 300
        endif
        if(cell(5).eq.0.0)then
          write(*,*)' !!! WARNING, space group is ambiguous !!!'
          write(*,*)
          write(*,*)' Hexagonal setting selected'
        endif
        klass=6
      else if(-146.eq.spnum)then
        klass=7
      else if(spnum.eq.149 .or. spnum.eq.151 .or. spnum.eq.153)then
        klass=8
      else if(spnum.eq.150 .or. spnum.eq.152 .or. spnum.eq.154)then
        klass=9
      else if(155.eq.spnum)then
        if(cell(5) .eq. cell(6) .and. cell(5).ne.0.0)then
          write(*,*)' !!! Warning, Rhombohedral setting selected !!!'
          nspgrp=nspgrp+1
          goto 300
        endif
        if(cell(5).eq.0.0)then
          write(*,*)' !!! WARNING, space group is ambiguous !!!'
          write(*,*)
          write(*,*)' Hexagonal setting selected'
        endif
        klass=9
      else if(-155 .eq. spnum)then
        klass=10
      else if(168.le.spnum .and. spnum.le.173)then
        klass=11
      else if(177.le.spnum .and. spnum.le.182)then
        klass=12
      else if(195.le.spnum .and. spnum.le.199)then
        klass=13
      else if(207.le.spnum .and. spnum.le.214)then
        klass=14
      else if(-5.le.spnum .and. spnum.le.-3)then
        if(cell(4).ne.90.0 .and. cell(4).ne.0.0)then
          write(*,*)' !!! Error, monoclinic A unique, not supported !!!'
          stop
        else if(cell(5).ne.90.0 .and. cell(5).ne.0.0 .and.
     $         (cell(6).eq.90.0 .or.  cell(6).eq.0.0))then
          write(*,*)' !!! Warning, monoclinic B unique selected !!!'
          nspgrp=nspgrp-1
          goto 300
        endif
        if(cell(6).eq.90.0 .or. cell(6).eq.0.0)then
          write(*,*)' !!! WARNING, space group is ambiguous !!!'
          write(*,*)
          write(*,*)' monoclinic with C-unique has been selected'
          write(*,*)' use the positive space group number if you wish'
          write(*,*)' to work with B-unique (e.g. 3, 4 or 5)'
        endif
        klass=16
      else
        write(*,*)' !!! Program bug in getsym, illegal class !!!'
        stop
      endif

c check validity of cell parameters if cell() not zero
      do 425,l=1,6
        if(cell(l).eq.0) goto 425
        if(celres(l,klass).lt.0)then
          if(cell(l).eq.cell(nint(-celres(l,klass)))) goto 425
        else
          if(cell(l).eq.celres(l,klass)) goto 425
        endif
        write(*,*)' !!! ERROR: illegal cell parameters !!!'
        write(*,*)
        write(*,'(  ''A    ='',f8.2,'' ; B   ='',f8.2,'' ; C    ='',
     $    f8.2,/,'' ; Alpha='',f8.2,'' ; Beta='',f8.2,'' ; Gamma='',
     $    f8.2)')cell
        write(*,*)
        if(celres(l,klass).lt.0.)then
          write(*,'(a,i1,a,i1,a)')' Cell parameter ',l,' and ',
     $      nint(-celres(l,klass)),' must be identical'
        else
          write(*,'(a,i1,a,f8.2)')' Cell parameter ',l,
     $      ' must be equal to',celres(l,klass)
        endif
        stop
  425 continue

c find first zero in ops-array to add the translational generators
      nop=0
  450 continue
        nop=nop+1
      if(ops(nop).ne.0) goto 450

c include centering generators if required
      centr = spgnam(1:1)
      nprim = 2
      if(centr.eq.'A')then
        ops(nop)=41
      else if(centr.eq.'B')then
        ops(nop)=42
      else if(centr.eq.'C')then
        ops(nop)=43
      else if(centr.eq.'I')then
        ops(nop)=44
      else if(centr.eq.'F')then
        ops(nop)=41
        ops(nop+1)=42
        nprim=4
      else if(centr.eq.'R'.and. .not.(klass.eq.7 .or. klass.eq.10))then
        ops(nop)=45
        nprim=3
      else
c no centering, explicitly set centr to 'P' for R3(2) rhombohedral
        centr='P'
        nprim=1
      endif

c initialize the identity matrix
      do 500,l=1,4
        do 400,m=1,4
          symops(m,l,1)=0.0
  400   continue
        symops(l,l,1)=1.0
  500 continue

      nop=1
      newsym=1
  600 if(ops(nop).ne.0)then
        do 800,m=0,2
          do 700,l=1,3
            symmat(m+1,l)=g(4*m+l,ops(nop))
            orimat(m+1,l)=g(4*m+l,ops(nop))
  700     continue
          symmat(m+1,4)=g(4*m+4,ops(nop))/24.0
          orimat(m+1,4)=g(4*m+4,ops(nop))/24.0
          symmat(4,m+1)=0.0
          orimat(4,m+1)=0.0
  800   continue
        symmat(4,4)=0.0
        orimat(4,4)=0.0

        nsym=newsym
  900   continue
        do 1000,l=1,nsym
          newsym=newsym+1
          call mult(symops(1,1,l),symmat,symops(1,1,newsym))
 1000   continue
        call mult(symmat,orimat,symmat)
        do 1200,m=1,4
          do 1100,l=1,3
            if(symops(l,m,1).ne.symmat(l,m)) goto 900
 1100     continue
 1200   continue

        nop=nop+1
        goto 600
      endif

      nprim = newsym/nprim
      nsym  = newsym

      write(*,*)
      write(*,'(a,i4)')'Space group number:',spnum
      write(*,'(a,x,a)')'Space group name  :',spgnam
      write(*,'(a,x,a)')'Point group name  :',pgnams(klass)
      write(*,'(a,x,a)')'Crystal system    :',cr_syst(klass)
      write(*,'(a,x,a)')'Centering type    :',centr

      cosa = cos(deg2rad*cell(4))
      cosb = cos(deg2rad*cell(5))
      cosg = cos(deg2rad*cell(6))
      celvol = cell(1) * cell(2) * cell(3) *
     $  sqrt(1 - cosa**2 - cosb**2 - cosg**2 + 2*cosa*cosb*cosg)
      if(celvol.gt.0.0)then
        write(*,'(A,F10.0,A)')'Unit cell volume  :',celvol,' A**3'
      endif

      write(*,*)
      write(*,'(a,i3)')'Number of symmetry operations :',nsym
      write(*,'(a,i3)')'Number of primitive operations:',nprim
      write(*,*)

      if(format.eq.'GENERIC')then
        do 1250,l=1,nsym
          call symlst(symops(1,1,l),frmat,format)
          m=index(frmat,')')
          frmat(m:m)=' '
          write(*,*)frmat(2:80)
 1250   continue
      else if(format.eq.'SYMTR')then
        do 1400,m=2,nsym
          do 1300,l=1,3
            write(*,'(''SYMTR'',i1,1x,i3,3f10.7,5x,f10.7)')
     $        l,m-1,(symops(l,n,m),n=1,4)
 1300     continue
 1400   continue
      else if(format.eq.'XPLOR')then
        write(*,'(''A    ='',f8.3,''  B   ='',f8.3,''  C    ='',f8.3,/,
     $            ''ALPHA='',f8.3,''  BETA='',f8.3,''  GAMMA='',f8.3)')
     $    cell
        write(*,*)
        do 1500,l=1,nsym
          call symlst(symops(1,1,l),frmat,format)
          write(*,'(a)')'SYMMETRY = '//frmat
 1500   continue
      else
        write(*,*)' Not yet implemented, keep complaining'
      endif

      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      subroutine symlst(symop,symdes,style)

c subroutine that takes a symmetry operation as a 4x4 matrix and returns
c the description for the new x, y and z coordinates in the requested
c style

      implicit none

c argument variables
      character symdes*80,style*20
      real symop(4,4)
      integer k, l, nc

      if(style.eq.'SYMTR')then
c not yet implemented
      else if(style(1:5).eq.'XPLOR' .or.
     $        style(1:7).eq.'GENERIC')then
        symdes='( '
        nc=3
        do 200,k=1,3
          do 100,l=1,4
            if(symop(k,l).eq.0.0)then
c do nothing
            else if(symop(k,l).eq.-1.0)then
              symdes(nc:nc+1)='-'//char(ichar('W')+l)
              nc=nc+2
            else if(symop(k,l).eq. 1.0)then
              if(index('(,',symdes(nc-2:nc-2)).eq.0)then
                symdes(nc:nc+1)='+'//char(ichar('W')+l)
                nc=nc+2
              else
                symdes(nc:nc)=char(ichar('W')+l)
                nc=nc+1
              endif
            else if(nint(12.0*symop(k,l)).eq. 2.0)then
              symdes(nc:nc+3)='+1/6'
              nc=nc+4
            else if(nint(12.0*symop(k,l)).eq. 3.0)then
              symdes(nc:nc+3)='+1/4'
              nc=nc+4
            else if(nint(12.0*symop(k,l)).eq. 4.0)then
              symdes(nc:nc+3)='+1/3'
              nc=nc+4
            else if(nint(12.0*symop(k,l)).eq. 6.0)then
              symdes(nc:nc+3)='+1/2'
              nc=nc+4
            else if(nint(12.0*symop(k,l)).eq. 8.0)then
              symdes(nc:nc+3)='+2/3'
              nc=nc+4
            else if(nint(12.0*symop(k,l)).eq. 9.0)then
              symdes(nc:nc+3)='+3/4'
              nc=nc+4
            else if(nint(12.0*symop(k,l)).eq. 10.0)then
              symdes(nc:nc+3)='+5/6'
              nc=nc+4
            else
              write(*,*)' Illegal matrix element',symop(k,l)
              stop
            endif
  100     continue
          symdes(nc:nc+1)=', '
          nc=nc+2
  200   continue
        nc=nc-2
        symdes(nc:nc+1)=' )'
      else
        write(*,*)' Illegal symmetry format'
        stop
      endif

      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      function input(card,option,frmat)

c Function input reads the first word from the character array card. A
c word is a consecutive sequence of non-word brakers " ,()<>=[]". This
c word is passed to the calling routine in character array option. The
c data format is returned in character array frmat. frmat can be
c used to read the data into a variable in the calling subroutine and
c also allows the calling routine to check if the data type is legal.
c The function returns as a logical. True if a word was successfully
c read or false if an end-of-input occured.
c Please note that the last position of the card is reserved for
c end-of-input detection.
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c This subroutine is part of the SFTOOLS program.   Bart Hazes
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      implicit none

c local declarations
      character*(*) card, option, frmat
      character*1   wrdtyp, char
      integer       l, nstart, nc, nquote, nchar
      logical       input

      option = ' '
      frmat  = ' '
      nchar  = len(card)

c skip leading white space
      do 100,l=1,nchar
        if(card(l:l).ne.' ')then
          nstart=l
          goto 200
        endif
  100 continue
c if you reach this point then end-of-input is reached
      input=.false.
      return

  200 continue

c now copy the next word to option and set the card characters to space
c nc counts the number of characters in the word
c wrdtyp stores the type of data (integer 'I', real 'F' or character 'A')

      nc=0
      nquote=0
      char=card(nstart:nstart)
      if(char.eq.'"')then
c this is a string
        wrdtyp='A'
        do 300,l=nstart+1,nchar
          char=card(l:l)
          if(char.eq.'"')then
            nquote=2
            goto 500
          endif
          nc=nc+1
          option(nc:nc)=char
  300   continue
        write(*,'(a)')'Closing quote is missing'
        input=.false.
        return
      else if(char.eq.'<' .and.
     $        index('>=',card(nstart+1:nstart+1)).ne.0)then
c < is a wordbraker unless it is part of a <> or <= construct
        wrdtyp='A'
        nc=nc+1
        option(nc:nc)=char
        nc=nc+1
        option(nc:nc)=card(nstart+1:nstart+1)
        card(nstart+1:nstart+1)=' '
      else if(char.eq.'>' .and. card(nstart+1:nstart+1).eq.'=')then
c > is a wordbraker unless it is part of a >= construct
        wrdtyp='A'
        nc=nc+1
        option(nc:nc)=char
        nc=nc+1
        option(nc:nc)=card(nstart+1:nstart+1)
        card(nstart+1:nstart+1)=' '
      else if(index(',()<>=[]',char).ne.0)then
c the first character is a word braker, return the wordbraker
        wrdtyp='A'
        nc=nc+1
        option(nc:nc)=char
      else
c normal number or word. Set default type to integer
        wrdtyp='I'
c move characters to option until wordbraker is found
        do 400,l=nstart,nchar
          char=card(l:l)
c is it a word braker? yes, then our word is ready
          if(index(' ,()<>=[]',char).ne.0) goto 500
c no then store this character and set card positon to space
          nc=nc+1
          option(nc:nc)=char
          card(l:l)=' '
c if it has a non-integer character, then it may be a floating point number
          if(index('+-0123456789',char).eq.0 .and. wrdtyp.eq.'I')
     $      wrdtyp='F'
c if it has a non-float character, then it must be a character string
          if(index('+-.Ee0123456789',char).eq.0) wrdtyp='A'
  400   continue
c if you reach this point then that acts as a wordbraker
      endif

  500 continue
c now create the fortran format string to read the word
c e.g. to read the real -0.345E-03 the format is (F10).

      if(wrdtyp.eq.'F' .and. nc.eq.1)then
c the user typed . E or e as input. It is unlikely that this is meant to
c represent zero, so set wrdtyp to 'A'
        wrdtyp='A'
      endif

      if(wrdtyp.eq.'I' .and. nc.eq.1 .and.
     $   index('-+', option(1:1)) .ne. 0)then
c the user typed + or - as input. It is unlikely that this is meant to
c represent zero, so set wrdtyp to 'A'
        wrdtyp='A'
      endif

      if(wrdtyp.eq.'F')then
c VAX wants an explicit decimal point specification in real formats.
c However it does not have to be correct. So just add .0
        if(nc.lt.10)then
          write(frmat,'(2a1,i1,a3)')'(',wrdtyp,nc,'.0)'
        else
          write(frmat,'(2a1,i2,a3)')'(',wrdtyp,nc,'.0)'
        endif
      else
        if(nc.lt.10)then
          write(frmat,'(2a1,i1,a1)')'(',wrdtyp,nc,')'
        else
          write(frmat,'(2a1,i2,a1)')'(',wrdtyp,nc,')'
        endif
      endif

c reset card so that it starts with the next word
      nstart=nstart+nc+nquote
      if(card(nstart:nstart) .eq. ' ')then
        nstart = nstart + 1
        goto 600
      endif

  600 card=card(nstart:nchar)

      input=.true.

      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      real function gt_real(card)

c function that tries to interpret the first word on the input card as
c a real. The real value is returned as the function value. A value of
c -1.0E+11 is returned in case of an error
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c This subroutine is part of the SFTOOLS program.   Bart Hazes
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      implicit none

c local variables
      character     option*80, frmat*80
      character*(*) card
      integer       ival
      logical       input

      gt_real = -1.0E+11

  100 if(input(card,option,frmat))then
        if(frmat(2:2) .eq. 'I')then
          read(option,frmat)ival
          gt_real = real(ival)
          return
        else if(frmat(2:2) .eq. 'F')then
          read(option,frmat)gt_real
          return
        else
          call upper(option)
          if(option .eq. 'MNF')then
          else if(option .eq. 'ABSENT')then
            gt_real = -1.0E+10
            return
          else if(option .eq. 'TOOWEAK')then
            gt_real = -1.0E+10
            return
          else if(option .eq. 'TOOSTRONG')then
            gt_real = 1.0E+10
            return
          else
            write(*,'(A)')'Real value expected but could not interpret'
            write(*,'(A)')'the following string:'//option
            call push(option, card)
          endif
        endif
      else
        write(*,'(A)')'No value given when real expected'
      endif

      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      subroutine push(card1,card2)
c subroutine to merge card1 and card2. The result is returned in card2.
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c This subroutine is part of the SFTOOLS program.   Bart Hazes
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      implicit none

c local variables
      character*(*) card1, card2
      integer       len1, last1, len2, last2, j

      len1  = len(card1)
      last1 = 0
      do j=len1,1,-1
        if(card1(j:j).ne.' ' .and. last1.eq.0) last1=j
      enddo

      len2  = len(card2)
      last2 = 0
      do j=len2,1,-1
        if(card2(j:j).ne.' ' .and. last2.eq.0) last2=j
      enddo

      if(last1+last2 .lt. len2)then
        card2=card1(1:last1+1)//card2(1:last2)
      else
        card2=card1(1:last1+1)//card2(1:len2-1-last1)
        write(*,'(A)')'Overflow in subroutine PUSH'
        stop
      endif

      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      subroutine upper(string)
c subroutine that converts all characters in 'string' to upper case
c numbers, punctuation characters etc are left unchanged
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c This subroutine is part of the SFTOOLS program.   Bart Hazes
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      implicit none

c local declarations
      character*(*) string
      character*26  azlow, azupp
      integer       j, ind

      data azlow / 'abcdefghijklmnopqrstuvwxyz' /
      data azupp / 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' /

      do j=1,len(string)
        ind = index(azlow,string(j:j))
        if(ind .ne. 0) string(j:j) = azupp(ind:ind)
      enddo

      end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      subroutine mult(m1,m2,m4)

      implicit none

      real m1(4,4), m2(4,4), m3(4,4), m4(4,4)
      integer j,k,l

      do 20 j=1,4
        do 10 k=1,4
          m3(k,j) = 0.0
   10   continue
   20 continue

      do 300,j=1,3
        do 200,k=1,3
          do 100,l=1,4
            m3(j,l)=m3(j,l)+m1(k,l)*m2(j,k)
  100     continue
  200   continue
        m3(j,4)=m3(j,4)+m2(j,4)
c bring translational part in the 0 to 1 interval
        m3(j,4) = real(mod( nint(m3(j,4)*24.0) + 2400, 24))/24.0
  300 continue

      do 500,j=1,4
        do 400,k=1,3
          m4(k,j)=m3(k,j)
  400   continue
  500 continue

      end
